@@ -612,13 +612,14 @@ void createMask(NET net, u_char slack, u_char halo)
   int i, j, orient;
   int dx, dy, gx1, gx2, gy1, gy2;
   int xcent, ycent, xmin, ymin, xmax, ymax;
+  int oxmin, oymin, oxmax, oymax;
 
   fillMask((u_char)halo);
 
-  xmin = net->xmin;
-  xmax = net->xmax;
-  ymin = net->ymin;
-  ymax = net->ymax;
+  oxmin = net->xmin;
+  oxmax = net->xmax;
+  oymin = net->ymin;
+  oymax = net->ymax;
 
   xcent = net->trunkx;
   ycent = net->trunky;
@@ -631,8 +632,10 @@ void createMask(NET net, u_char slack, u_char halo)
      // Horizontal trunk
      orient |= 1;
 
-     ycent = analyzeCongestion(net->trunky, ymin, ymax, xmin, xmax);
+     ycent = analyzeCongestion(net->trunky, oymin, oymax, oxmin, oxmax);
      ymin = ymax = ycent;
+     xmin = oxmin;
+     xmax = oxmax;
 
      for (i = xmin - slack; i <= xmax + slack; i++) {
 	if (i < 0 || i >= NumChannelsX[0]) continue;
@@ -667,6 +670,8 @@ void createMask(NET net, u_char slack, u_char halo)
      // Vertical trunk
      orient |= 2;
      xmin = xmax = xcent;
+     ymin = oymin;
+     ymax = oymax;
 
      for (i = xcent - slack; i <= xcent + slack; i++) {
 	if (i < 0 || i >= NumChannelsX[0]) continue;
@@ -704,9 +709,9 @@ void createMask(NET net, u_char slack, u_char halo)
      dtap = (n1->taps == NULL) ? n1->extend : n1->taps;
      if (!dtap) continue;
 
-     if (orient | 1) 	// Horizontal trunk, vertical branches
+     if (orient & 1) 	// Horizontal trunk, vertical branches
 	create_vbranch_mask(n1->branchx, n1->branchy, ycent, slack, halo);
-     if (orient | 2) 	// Vertical trunk, horizontal branches
+     if (orient & 2) 	// Vertical trunk, horizontal branches
 	create_hbranch_mask(n1->branchy, n1->branchx, xcent, slack, halo);
   }
 
@@ -715,7 +720,7 @@ void createMask(NET net, u_char slack, u_char halo)
   // the branch end that is closer to the trunk and the branch that
   // is its nearest neighbor.
 
-  if (orient | 1) {	// Horizontal trunk, vertical branches
+  if (orient & 1) {	// Horizontal trunk, vertical branches
      for (n1 = net->netnodes; n1; n1 = n1->next) {
 	for (n2 = net->netnodes->next; n2; n2 = n2->next) {
 
@@ -740,7 +745,7 @@ void createMask(NET net, u_char slack, u_char halo)
         }
      }
   }
-  if (orient | 2) {		// Vertical trunk, horizontal branches
+  if (orient & 2) {		// Vertical trunk, horizontal branches
      for (n1 = net->netnodes; n1; n1 = n1->next) {
 	for (n2 = net->netnodes->next; n2; n2 = n2->next) {
 
@@ -777,7 +782,7 @@ void createMask(NET net, u_char slack, u_char halo)
   if (Verbose > 2) {
      if (net->numnodes == 2)
         Fprintf(stdout, "Two-port mask has bounding box (%d %d) to (%d %d)\n",
-			xmin, ymin, xmax, ymax);
+			oxmin, oymin, oxmax, oymax);
      else
         Fprintf(stdout, "multi-port mask has trunk line (%d %d) to (%d %d)\n",
 			xmin, ymin, xmax, ymax);
