@@ -58,9 +58,9 @@ typedef int (*__compar_fn_t)(const void*, const void*);
 
 // define possible gate orientations
 
-#define  MNONE   0
-#define  MX      1
-#define  MY      2
+#define MNONE	0
+#define MX	1
+#define MY	2
 
 // define search directions
 
@@ -76,6 +76,15 @@ typedef int (*__compar_fn_t)(const void*, const void*);
 #define VIA_PATTERN_NORMAL	 0
 #define VIA_PATTERN_INVERT	 1
 
+// linked list structure for holding a list of char * strings
+
+typedef struct linkedstring_ *LinkedStringPtr;
+
+typedef struct linkedstring_ {
+   char *name;
+   LinkedStringPtr next;
+} LinkedString;
+
 // structure holding input and output scalefactors
 
 typedef struct scalerec_ {
@@ -235,6 +244,8 @@ struct nodeinfo_ {
 #define NI_OFFSET_NS	 0x04	// Tap offset north(+)/south(-)
 #define NI_OFFSET_EW	 0x08	// Tap offset east(+)/west(-)
 #define NI_OFFSET_MASK   0x0c	// Tap offset mask (N/S + E/W)
+#define NI_NO_VIAX   	 0x10	// Via in ViaX array is prohibited
+#define NI_NO_VIAY   	 0x20	// Via in ViaY array is prohibited
 
 struct node_ {
   NODE    next;
@@ -470,6 +481,7 @@ void   read_def(char *filename);
 
 #ifdef TCL_QROUTER
 int    write_delays(char *filename);
+int    write_spef(char *filename);
 #endif
 
 int    dofirststage(u_char graphdebug, int debug_netnum);
