@@ -32,6 +32,7 @@ Tcl_Interp *qrouterinterp;
 Tcl_Interp *consoleinterp;
 
 int stepnet = -1;
+int batchmode = 0;
 
 /* Command structure */
 
@@ -511,9 +512,10 @@ int
 Qrouter_Init(Tcl_Interp *interp)
 {
    int cmdidx;
-   Tk_Window tktop;
    char command[256];
    char version_string[20];
+   Tk_Window tktop;
+   char *nullgvar;
 
    /* Interpreter sanity checks */
    if (interp == NULL) return TCL_ERROR;
@@ -521,13 +523,25 @@ Qrouter_Init(Tcl_Interp *interp)
    /* Remember the interpreter */
    qrouterinterp = interp;
 
-   if (Tcl_InitStubs(interp, "8.1", 0) == NULL) return TCL_ERROR;
+   if (Tcl_InitStubs(interp, "8.5", 0) == NULL) return TCL_ERROR;
 
    strcpy(command, "qrouter::");
    
-   /* Create the start command */
+   /* NOTE:  Qrouter makes calls to Tk routines that may or may not	*/
+   /* exist, depending on whether qrouter was called with or without	*/
+   /* graphics.  We depend on the Tcl/Tk stubs methods to allow 	*/
+   /* qrouter to run without linking to Tk libraries.			*/
 
-   tktop = Tk_MainWindow(interp);
+   nullgvar = (char *)Tcl_GetVar(interp, "no_graphics_mode", TCL_GLOBAL_ONLY);
+   if ((nullgvar == NULL) || !strcasecmp(nullgvar, "false")) {
+      if (Tk_InitStubs(interp, "8.5", 0) == NULL) return TCL_ERROR;
+      tktop = Tk_MainWindow(interp);
+      batchmode = 0;
+   }
+   else {
+      tktop = NULL;
+      batchmode = 1;
+   }
 
    /* Create all of the commands (except "simple") */
 
@@ -538,11 +552,11 @@ Qrouter_Init(Tcl_Interp *interp)
 		(ClientData)tktop, (Tcl_CmdDeleteProc *) NULL);
    }
 
-   /* Command which creates a "simple" window.		*/
-
-   Tcl_CreateObjCommand(interp, "simple",
+   if (tktop != NULL) {
+      Tcl_CreateObjCommand(interp, "simple",
 		(Tcl_ObjCmdProc *)Tk_SimpleObjCmd,
 		(ClientData)tktop, (Tcl_CmdDeleteProc *) NULL);
+   }
 
    Tcl_Eval(interp, "lappend auto_path .");
 
@@ -577,7 +591,7 @@ qrouter_start(ClientData clientData, Tcl_Interp *interp,
     char **argv;
 
     /* For compatibility with the original C code, convert Tcl	*/
-    /* object arguments to strings.  Break out "-s <name>",	*/
+    /* object arguments to strings.  Handle "-s <name>",	*/
     /* which is not handled by runqrouter(), and source the	*/
     /* script <name> between runqrouter() and read_def().	*/
 
@@ -585,22 +599,45 @@ qrouter_start(ClientData clientData, Tcl_Interp *interp,
     argc = 0;
     for (i = 1; i < objc; i++) {
 	if (!strcmp(Tcl_GetString(objv[i]), "-s"))
-	    scriptfile = strdup(Tcl_GetString(objv[++i]));
-	else
-	    argv[argc++] = strdup(Tcl_GetString(objv[i]));
+	    scriptfile = strdup(Tcl_GetString(objv[i + 1]));
+	argv[argc++] = strdup(Tcl_GetString(objv[i]));
     }
 
     result = runqrouter(argc, argv);
-    if (result == 0) GUI_init(interp);
+    if ((result == 0) && (batchmode == 0)) GUI_init(interp);
 
     for (i = 0; i < argc; i++)
         free(argv[i]);
     free(argv);
 
     if (scriptfile != NULL) {
-	result = Tcl_EvalFile(interp, scriptfile);
+
+	/* First check that the script file exists.  If not,	*/
+	/* then generate an error here.				*/
+
+	FILE *scriptf = fopen(scriptfile, "r");
+	if (scriptf == NULL) {
+	    Fprintf(stderr, "Script file \"%s\" unavaliable or unreadable.\n",
+			scriptfile);
+	    Tcl_SetResult(interp, "Script file unavailable or unreadable.", NULL);
+	    result = TCL_ERROR;
+	}
+	else {
+	    fclose(scriptf);
+	    result = Tcl_EvalFile(interp, scriptfile);
+	}
 	free(scriptfile);
-	if (result != TCL_OK) return result;
+
+	/* The script file should determine whether or not to	*/
+	/* exit by including the "quit" command.  But if there	*/
+	/* is an error in the script, then always quit.		*/
+
+	if (result != TCL_OK) {
+	    /* Make sure Tcl has generated all output */
+	    while (Tcl_DoOneEvent(TCL_DONT_WAIT) != 0);
+	    /* And exit gracefully */
+	    qrouter_quit(clientData, interp, 1, objv);
+	}
     }
 
     if ((DEFfilename != NULL) && (Nlgates == NULL)) {
@@ -1288,6 +1325,10 @@ qrouter_stage3(ClientData clientData, Tcl_Interp *interp,
     if (net == NULL)
 	failcount = dothirdstage(dodebug, stepnet, effort);
     else {
+        /* To do:  Duplicate behavior of dothirdstage(), which	*/
+	/* is to retain the original route solution and restore	*/
+	/* it in case the routing fails.			*/
+
 	if ((net != NULL) && (net->netnodes != NULL)) {
 	    result = doroute(net, (u_char)0, dodebug);
 	    failcount = (result == 0) ? 0 : 1;
@@ -1363,7 +1404,7 @@ qrouter_remove(ClientData clientData, Tcl_Interp *interp,
 	    case AllIdx:
 		for (i = 0; i < Numnets; i++) {
 		   net = Nlnets[i];
-		   ripup_net(net, (u_char)1, (u_char)1);
+		   ripup_net(net, (u_char)1, (u_char)1, (u_char)0);
 		}
 		draw_layout();
 		break;
@@ -1371,7 +1412,7 @@ qrouter_remove(ClientData clientData, Tcl_Interp *interp,
 		for (i = 2; i < objc; i++) {
 		    net = LookupNet(Tcl_GetString(objv[i]));
 		    if (net != NULL)
-			ripup_net(net, (u_char)1, (u_char)1);
+			ripup_net(net, (u_char)1, (u_char)1, (u_char)0);
 		}
 		draw_layout();
 		break;
@@ -1959,23 +2000,31 @@ qrouter_layerinfo(ClientData clientData, Tcl_Interp *interp,
 /*	  the rotation is swapped relative to the grid	*/
 /*	  positions used in the non-inverted case.	*/
 /*							*/
+/* use: List of names of vias to use.  If any via not	*/
+/*	in this list is found when reading a .lef file	*/
+/*	it will be ignored.				*/
+/*							*/
 /* Options:						*/
 /*							*/
 /*	via stack [none|all|<value>]			*/
 /*	via pattern [normal|inverted]			*/
+/*	via use <via_name> [<via_name> ...]		*/
 /*------------------------------------------------------*/
 
 static int
 qrouter_via(ClientData clientData, Tcl_Interp *interp,
             int objc, Tcl_Obj *CONST objv[])
 {
-    int idx, idx2, result, value;
+    int idx, idx2, result, value, i;
+    char *vname;
+    Tcl_Obj *lobj;
+    LinkedStringPtr viaName, newVia;
 
     static char *subCmds[] = {
-	"stack", "pattern", NULL
+	"stack", "pattern", "use", NULL
     };
     enum SubIdx {
-	StackIdx, PatternIdx
+	StackIdx, PatternIdx, UseIdx
     };
    
     static char *stackSubCmds[] = {
@@ -1986,7 +2035,7 @@ qrouter_via(ClientData clientData, Tcl_Interp *interp,
     };
 
     static char *patternSubCmds[] = {
-	"none", "normal", "invert", NULL
+	"none", "normal", "inverted", NULL
     };
     enum patternSubIdx {
 	PatNoneIdx, PatNormalIdx, PatInvertIdx
@@ -2008,6 +2057,15 @@ qrouter_via(ClientData clientData, Tcl_Interp *interp,
 				Tcl_NewStringObj(
 				patternSubCmds[ViaPattern + 1], -1));
 		    break;
+		case UseIdx:
+		    /* Return list of vias to use */
+		    lobj = Tcl_NewListObj(0, NULL);
+		    for (viaName = AllowedVias; viaName; viaName = viaName->next) {
+			Tcl_ListObjAppendElement(interp, lobj,
+				Tcl_NewStringObj(viaName->name, -1));
+		    }
+		    Tcl_SetObjResult(interp, lobj);
+		    break;
 	    }
 	}
 	else {
@@ -2041,6 +2099,24 @@ qrouter_via(ClientData clientData, Tcl_Interp *interp,
 			return result;
 		    ViaPattern = idx2 - 1;
 		    break;
+		case UseIdx:
+		    /* Create list of vias to use */
+		    for (i = 2; i < objc; i++) {
+			vname = Tcl_GetString(objv[i]);
+			/* First check if name is in list already */
+			for (viaName = AllowedVias; viaName; viaName = viaName->next) {
+			    if (!strcmp(vname, viaName->name))
+				break;
+			}	
+			if (viaName != NULL) continue;
+			newVia = (LinkedStringPtr)malloc(sizeof(LinkedString));
+			newVia->name = strdup(vname);
+			newVia->next = AllowedVias;
+			AllowedVias = newVia;
+		    }
+		    /* Regenerate the ViaX and ViaY lists */
+		    LefAssignLayerVias();
+		    break;
 	    }
 	}
     }
